/*
 * Netris -- A free networked version of T*tris
 * Copyright (C) 1994,1995,1996  Mark H. Weaver <mhw@netris.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define NOEXT
#include "netris.h"

#include <stdlib.h>
#include <stdbool.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>

#include "client.h"
#include "util.h"
#include "board.h"
#include "curses.h"
#include "inet.h"
#include "msg.h"

static struct option options[] = {
	{ "ascii",		2, 0, 'a' },
	{ "connect",	1, 0, 'c' },
	{ "port",		1, 0, 'p' },
	{ "level",		1, 0, 'l' },
	{ "nick",		1, 0, 'n' },
	{ "team",		1, 0, 't' },
	{ "color",		2, 0, 'C' },
	{ "slowterm",	2, 0, 'S' },
	{ "keys",		1, 0, 'k' },
	{ "info",		0, 0, 'H' },
	{ "help",		0, 0, 'h' },
	{ 0,			0, 0,  0 }
};

enum {
	KT_left, KT_right, KT_rotright, KT_rotleft, KT_drop, KT_dropsoft, KT_down,
	KT_faster, KT_pause, KT_redraw, KT_say, KT_quit, KT_numKeys
};

static char *keyNames[KT_numKeys+1] = {
	"Left", "Right", "RotRight", "RotLeft", "Drop", "DropSoft", "Down",
	"Faster", "Pause", "Redraw", "Say", "Quit", NULL
};

sets_t Sets = {3, 0, 1, 1, 1};

static char keyTable[KT_numKeys+1];

enum {
	CT_quit, CT_pause,
	CT_MAX
};
static char *cmds[] = {
	"quit", "pause"
};

static char *hostStr;
static bool paused = 0;
static char lastadd;


static void handle_setkeys(char *newKeys)
{
	int i, k, ch;
	char used[256];
	int errs = 0;
	char scratch[6];

	/* XXX assumptions about ASCII encoding here */
	for (i = k = 0; newKeys[i] && k < KT_numKeys; i++,k++) {
		if (newKeys[i] == '^' && newKeys[i+1])
			keyTable[k] = toupper(newKeys[++i]) - ('A' - 1);
		else
			keyTable[k] = newKeys[i];
	}
	memset(used, 0, sizeof(used));
	for (k = 0; k < KT_numKeys; k++) {
		ch = (unsigned char) keyTable[k];
		if (used[ch]) {
			if (iscntrl(ch) && ch < ' ')
				sprintf(scratch, "Ctrl-%c", ch + ('A' - 1));
			else if (isprint(ch))
				sprintf(scratch, "\"%c\"", ch);
			else
				sprintf(scratch, "0x%X", ch);
			if (!errs)
				fprintf(stderr, "Duplicate key mappings:\n");
			errs++;
			fprintf(stderr, "  %s mapped to both %s and %s\n",
				scratch, keyNames[used[ch]-1], keyNames[k]);
		}
		used[ch] = k + 1;
	}
	if (errs)
		exit(1);
}

static void Usage(void)
{
	Header();
	fprintf(stderr,
		"Usage: netris <options>\n"
		"\n"
		"  -h, --help\t\tPrint this usage information\n"
		"  -H, --info\t\tShow distribution and warranty information\n"
		"\n"
		"  -S, --slowterm\tDisable inverse/bold/color for slow terminals\n"
		"  -a, --ascii\t\tUse ascii characters\n"
		"  -C, --color=0\t\tDisable color\n"
		"\n"
		"  -c, --connect <host>\tInitiate connection\n"
		"  -p, --port <port>\tSet port number (default is %d)\n"
		"\n"
		"  -t, --team <team>\tJoin a team (don't receive lines from your teammates)\n"
		"  -l, --level <lvl>\tBegin at a higher level (can be used as handicap)\n"
		"  -k, --keys <keys>\tRemap keys (default is \"%s\" for cursors)\n"
		"\n",
		DEFAULT_PORT, DEFAULT_KEYS
	);
}

static void handle_arg(char tag, char *value)
{
	switch (tag) {
	case 'a':	//ascii
		Sets.ascii = value && !strcasecmp(value, "0") ? 0 : 1;
		Sets.drawstyle &= ~Sets.ascii;
		break;
	case 'c':	//connect
		Game.type = GT_classicTwo;
		hostStr = value;
		break;
	case 'p':	//port
		port = atoi(value);
		break;
	case 'i':	//speed (of level 1)
		Game.initspeed = atof(value) * 1e6;
		break;
	case 'l':	//level
		Players[0].score.level = MIN(MAX(atof(value), 1), 15);
		break;
	case 'n':	//nick
		memcpy(Players[0].name, value, strlen(value) + 1);
		break;
	case 't':	//team
		Players[0].team = atoi(value);
		break;
	case 'C':	//color
		Sets.color = value && strcasecmp(value, "0") ? 1 : 0;
		break;
	case 'S':	//slowterm
		Sets.standout = value && !strcasecmp(value, "0") ? 1 : 0;
		break;
	case 'k':	//keys
		handle_setkeys(value);
		break;
	case 'H':	//info
		Header();
		DistInfo();
		exit(0);
	case 'h':	//help
		Usage();
		exit(0);
	default:
		Usage();
		exit(1);
	}
}

static void handle_conffile(char *filename)
{
	FILE *file_in;
	char buf[513];
	int i;
	char *ch;
	char tag[81], value[81];

	file_in = fopen(filename, "r");
	if (file_in) {
		while (fgets(buf, 512, file_in) != NULL) {
			if ((ch = strchr(buf, '#')))
				*ch = '\0'; // truncate string from # char
			for (i = strlen(buf) - 1; i >= 0; i--)
				if (buf[i] == ' ' || buf[i] == '\n' || buf[i] == '\t' || buf[i] == 13)
					buf[i] = '\0';
				else break;

			sscanf(buf, "%80[^= \t] = %80[^\n]", tag, value);
			for (i = 0; options[i].name; i++){
				if (!strcasecmp(options[i].name, tag)) {
					handle_arg(options[i].val, value);
					break;
				}
			}
		}
		fclose(file_in);
	} //read file
	else {
		fprintf(stderr, "Unable to open config file %s.\n", filename);
	} //defaults
}

static int player_piece(int scr, char shape)
{
	Players[scr].score.pieces++;
	{
		Players[scr].curShape = Players[scr].nextShape;
		Players[scr].nextShape = shape;
	}
	Players[scr].curY = Players[scr].boardVisible + 4;
	Players[scr].curX = Players[scr].boardWidth / 2 - 2;
	while (!shape_visible(Players[scr].curShape, scr,
	                     Players[scr].curY, Players[scr].curX))
		Players[scr].curY--;
	if (!shape_get(Players[scr].curShape, scr,
			Players[scr].curY, Players[scr].curX))
		return 0;
	shape_draw(Players[scr].curShape, scr,
		Players[scr].curY, Players[scr].curX, scr == me);
	return 1;
}

static void game_setpaused(void)
{ //check whether anyone paused the game
	int i;

	paused = Game.started < 1;
	for (i = 1; i < MAX_SCREENS; i++) if (Players[i].alive > 0)
		paused |= (Players[i].flags & SCF_paused) != 0;
}

static void game_reset(void)
{ //init new game
	int i;

	lastadd = me;
	SRandom(Game.seed);

	// speed = MAX(initspeed / SPEEDINC ** (level - 1), SPEEDMINIMUM)
	Game.speed = Game.initspeed;
	for (i = 1; i < Players[me].score.level; i++)
		Game.speed /= SPEEDINC;
	if (Game.speed < SPEEDMINIMUM)
		Game.speed = SPEEDMINIMUM;

	ResetBaseTime();  //reset timer
	SetITimer(Game.speed, Game.speed);
	Players[me].nextShape = ChooseOption(Game.shapes);
	for (i = 1; i <= maxPlayer; i++) {
		Players[i].score.score = Players[i].score.lines
		= Players[i].score.adds = 0;
		Players[i].score.pieces = -1;
		player_empty(i);
	} //reset all players
	screen_setup();
}

static void game_clear(int scr)
{ //check for full lines
	int linesCleared;
	int linevalues[] = { 40, 100, 400, 1200, }; //= 50*lines! - 10*(lines==1)
//	int linevaluesq[] = { 25, 50, 100, 200, 500, 720, 980, 1280, 1620, 2000,
//	                      2420, 2880, 3380, 3920, 4500, 5120, 5780, 6480 };
	int linevaluesq[] = { 20, 50, 100, 200, 500, 750, 1000, 1250, 1500, 2000,
	                      2500, 3000, 3500, 4000, 4500, 5000, 6000, 7500 };

	if ((linesCleared = player_lineclear(scr)) > 0) {
		if (Game.type == GT_onePlayer)
			if ((Players[scr].score.lines / 10) <
					((Players[scr].score.lines+linesCleared)/10)) {
				if ((Game.speed /= SPEEDINC) < SPEEDMINIMUM)
					Game.speed = SPEEDMINIMUM;
				SetITimer(Game.speed, SetITimer(0, 0));
				Players[scr].score.level++;
			} //level up
		Players[scr].score.score += Game.gravity
			? linevaluesq[linesCleared - 1] : linevalues[linesCleared - 1];
		Players[scr].score.lines += linesCleared;
		Players[scr].score.adds += linesCleared - (linesCleared < 4); //XXX match handicap
		if (scr == me) {
			if (Game.type == GT_classicTwo) {
				SendPacket(scr, NP_clear, 0, NULL);
				if (linesCleared > 1) {
					short junkLines;
					netint4 data[1];

					junkLines = linesCleared - (Game.gravity ? 1 : linesCleared < 4);
					data[0] = junkLines;
					SendPacket(me, NP_giveJunk, sizeof(data), data);
					msg_add("\\%d" MSG_GAME_LINES_SELF,
						Players[me].team > 7 ? 7 : Players[me].team, junkLines);
				} //send junk to others
			} //multiplayer
			else {
				msg_add("\\%d" MSG_GAME_LINE_SELF,
					Players[me].team > 7 ? 7 : Players[me].team, linesCleared);
			} //singleplayer
		} //IT'S YOU
	} //lines cleared
}

static void game_loop(void)
{
	bool changed = 0;
	short gameStatus = 2; //2=loop; 1=new piece; 0=quit
	bool chatMode = 0;
	char chatText[MSG_WIDTH] = "\0";

	void handle_cmd(char cmd, char *arg)
	{
		switch (cmd) {
		case CT_quit:
			window_msg_status(me);
			refresh();
			gameStatus = 0;
			return;
		case CT_pause:
			if (Players[me].alive <= 0) return;
			Players[me].flags ^= SCF_paused;
			if (Game.started > 1)
				msg_add(Players[me].flags & SCF_paused
					? MSG_GAME_PAUSE_SELF : MSG_GAME_UNPAUSE_SELF);
			else
				msg_add(Players[me].flags & SCF_paused
					? MSG_GAME_UNREADY_SELF : MSG_GAME_READY_SELF);
			game_setpaused();
			if (Game.type == GT_classicTwo)
				SendPacket(me, NP_pause, 0, NULL);
			window_msg_status(me);
			changed = 1;
			return;
		}
	}

	void handle_cmdstr(char *cmd)
	{
		char tag[17], value[81];
		char *cmdend;
		int i;

		if ((cmdend = strchr(cmd, ' '))) {
			*cmdend = 0;
		} else {
			cmdend = cmd + strlen(cmd); // whole string
		}
		for (i = 0; i < CT_MAX; i++){
			if (!strcasecmp(cmds[i], cmd)) {
				return handle_cmd(i, cmdend + 1);
			}
		}
		msg_add(MSG_CMD_EUNKNOWN, cmd);
	}

	void handle_str(char *str)
	{
		if (chatText[0] == '/') {
			if (chatText[1] != '/') {
				handle_cmdstr(chatText + 1);
				return;
			}
			memmove(chatText, chatText + 1, strlen(chatText));
		}

		msg_add("<\\%d%s\\7> %s",
			Players[me].team > 7 ? 7 : Players[me].team,
			Players[me].name, chatText);
		if (Game.type == GT_classicTwo)
			SendPacket(me, NP_msg, strlen(chatText) + 1, chatText);
	}

	void handle_key(char key)
	{
		char *p;

		if (chatMode) {
			if (key == 13) {
				// enter text
				chatMode = 0;
				if (chatText[0]) {
					handle_str(chatText);
					memset(chatText, 0, sizeof(chatText));
				} //say it
				else msg_add_char(27, -1, NULL); //escape
				return;
			}
			else if (key == 27) //escape
				chatMode = 0;
			else if (key == 127 && chatText) //backspace
				chatText[strlen(chatText) - 1] = 0;
			else if (strlen(chatText) < MSG_WIDTH-1) //text
				chatText[strlen(chatText)] = key;
			msg_add_char(key, strlen(chatText) - 1, chatText);
			return;
		} //key in chat mode

		if (!(p = strchr(keyTable, tolower(key)))) return;
		key = p - keyTable;

	void handle_keycmd(char key)
	{
		switch (key) {
		case KT_redraw:
			clear();
			screen_setup();
//			ScheduleFullRedraw();
			refresh();
			return;
		case KT_say:
			chatMode = 1;
			msg_add_char(key, strlen(chatText) - 1, chatText);
			return;
		case KT_quit:
			handle_cmd(CT_quit, NULL);
			return;
		case KT_pause:
			handle_cmd(CT_pause, NULL);
			return;
		}

		if (Players[me].alive <= 0 || paused) return;
		// actions only available while actually playing
		switch (key) {
		case KT_left:
			if (player_move(me, 0, -1) && spied) SendPacket(me, NP_left, 0, NULL);
			break;
		case KT_right:
			if (player_move(me, 0, 1) && spied) SendPacket(me, NP_right, 0, NULL);
			break;
		case KT_rotleft:
			if (player_rotate(me, -1) && spied) SendPacket(me, NP_rotleft, 0, NULL);
			break;
		case KT_rotright:
			if (player_rotate(me, 1) && spied) SendPacket(me, NP_rotright, 0, NULL);
			break;
		case KT_down:
			SetITimer(Game.speed, Game.speed);
			if (player_move(me, -1, 0)) {
				if (spied) SendPacket(me, NP_down, 0, NULL);
			} //move one down
			else
				gameStatus = 1; //completely dropped
			break;
		case KT_dropsoft:
			SetITimer(Game.speed, Game.speed);
			if (player_drop(me)) {
				if (spied) SendPacket(me, NP_drop, 0, NULL);
			}
			else gameStatus = 1; //dropped
			break;
		case KT_drop:
			SetITimer(Game.speed, Game.speed);
			if (player_drop(me)) {
				if (spied) SendPacket(me, NP_drop, 0, NULL);
			}
			gameStatus = 1; // drop
			break;
		case KT_faster:
			if (Game.type != GT_onePlayer) break;
			if ((Game.speed /= SPEEDINC) < SPEEDMINIMUM)
				Game.speed = SPEEDMINIMUM;
			SetITimer(Game.speed, SetITimer(0, 0));
			Players[me].score.level++;
			status_draw(me, Players[me].score);
			changed = 1;
			break;
		}
	}
		handle_keycmd(key);
	}

	int oldPaused = 0;

	void handle_net(_netEvent net)
	{
		switch(net.type) {
		case NP_newPiece:
		{
			memcpy(&Players[net.uid].nextShape, net.data,
				sizeof(Players[0].nextShape));
			player_piece(net.uid, Players[net.uid].curShape);
			break;
		}
		case NP_down:
			player_move(net.uid, -1, 0);
			break;
		case NP_left:
			player_move(net.uid, 0, -1);
			break;
		case NP_right:
			player_move(net.uid, 0, 1);
			break;
		case NP_rotleft:
			player_rotate(net.uid, -1);
			break;
		case NP_rotright:
			player_rotate(net.uid, 1);
			break;
		case NP_drop:
			player_drop(net.uid);
			break;
		case NP_clear:
			game_clear(net.uid);
			break;
		case NP_insertJunk:
		{
			netint4 data[3];

			memcpy(data, net.data, sizeof(data));
			player_lineadd(net.uid, Players[data[2]].team, data[0], data[1]);
			break;
		} //player added junklines
		case NP_giveJunk:
		{
			netint4 data[3];
			short column;

			if (Players[me].alive <= 0) break;
			memcpy(data, net.data, sizeof(data[0]));
			column = Random(0, Players[me].boardWidth);
			msg_add("\\%d" MSG_GAME_LINES,
				Players[net.uid].team > 7 ? 7 : Players[net.uid].team,
				Players[net.uid].name, data[0]);
			lastadd = net.uid;
			player_lineadd(me, Players[net.uid].team, data[0], column);
			if (spied) {
				data[1] = column;
				data[2] = net.uid;
				SendPacket(me, NP_insertJunk, sizeof(data), data);
			} //show changes to others
			break;
		} //receive junklines
		case NP_msg:
		{
			msg_add("<\\%d%s\\7> %s",
				Players[net.uid].team > 7 ? 7 : Players[net.uid].team,
				Players[net.uid].name, net.data, net.type);
			break;
		} //chat
		case NP_start:
		{
			int i;

			Game.started = 2;
			paused = 0;
			msg_add(MSG_GAME_START);
			for (i = 1; i < MAX_SCREENS; i++) if (Players[i].alive > 0)
				window_msg_status(i);
			break;
		} //start game
		case NP_stop:
		{
			if (Game.started > 1) {
				int winner;
				float timer;
				int i;

				msg_add(MSG_GAME_STOP);
				timer = CurTimeval() / 1e6;
				if (timer > 5) {
					for (i = MAX_SCREENS-1; i > 0; i--) if (Players[i].alive >= 0) {
						msg_add("\\%d%10s%6.1fp%5.1fa",
							Players[i].team > 7 ? 7 : Players[i].team, Players[i].name,
							Players[i].score.pieces / timer * 60,
							Players[i].score.adds / timer * 60);
						if (Players[i].alive > 0) winner = i;
					} //show player stats
				if (winner)
					msg_add(MSG_GAME_WINNER,
						Players[winner].name, timer / 60, (int)timer % 60);
				} //show game stats
				msg_add(NULL);
			} //game was playing
			Game.started = 0;
			memcpy(&Game.seed, net.data, net.size);
			{
				int i;

				for (i = 1; i < MAX_SCREENS; i++) if (Players[i].alive >= 0) {
					Players[i].alive = 1;
					Players[i].flags |= SCF_paused;
				} //reset players
			}
			game_reset();  //reset everything
			status_tick();    //redraw timer while unpaused
			game_setpaused(); //pause
			oldPaused = 0; //reset pause
			changed = 1;
			gameStatus = 1;
			break;
		} //stop game
		case NP_newPlayer:
		{
			if (net.uid>maxPlayer) maxPlayer = net.uid;
			memcpy(&Players[net.uid], net.data, net.size);
			player_empty(net.uid);
			screen_setup();
			if (Players[net.uid].team > 7)
				msg_add(MSG_GAME_JOIN, Players[net.uid].name);
			else
				msg_add(MSG_GAME_JOIN_TEAM, Players[net.uid].name,
					teamname[Players[net.uid].team]);
			if (Players[net.uid].flags & SCF_paused) {
				game_setpaused();
			} //player has paused
//			window_draw(net.uid);
//			window_msg_status(net.uid);
			changed = 1;
			break;
		} //player joined
		case NP_pause:
		{
			char s[20];

			Players[net.uid].flags ^= SCF_paused;
			if (Game.started > 1)
				strcpy(s, Players[net.uid].flags&SCF_paused
					? MSG_GAME_PAUSE : MSG_GAME_UNPAUSE);
			else
				strcpy(s, Players[net.uid].flags&SCF_paused
					? MSG_GAME_UNREADY : MSG_GAME_READY);
			msg_add(s, Players[net.uid].name);
			game_setpaused();
			window_msg_status(net.uid);
			changed = 1;
			break;
		} //(un)pause
		case NP_part:
			// player left
			game_setpaused();
			oldPaused = 0;
			Players[net.uid].alive = -1;
			msg_add(MSG_GAME_PART, Players[net.uid].name);
			game_setpaused();
			window_msg_status(net.uid);
			changed = 1;
			break;
		case NP_argghhh:
		{
			char i;
			memcpy(&i, net.data, sizeof(i));
			Players[net.uid].alive = 0;
			if (i == me)
				msg_add("\\%d" MSG_GAME_KILL_1_2,
					Players[me].team > 7 ? 7 : Players[me].team, Players[net.uid].name);
			else if (i == net.uid)
				msg_add("\\%d" MSG_GAME_KILL_2_2,
					Players[i].team > 7 ? 7 : Players[i].team, Players[i].name);
			else
				msg_add("\\%d" MSG_GAME_KILL_2_3,
					Players[i].team > 7 ? 7 : Players[i].team, Players[i].name,
					Players[net.uid].name);
			game_setpaused();
			window_msg_status(net.uid);
			changed = 1;
			break;
		} //G/O
		default:
			break;
		}
	}

	MyEvent event;
	long pauseTimeLeft;
	int i;

	game_reset();
	while (gameStatus) {
		gameStatus = 2;
		if (Players[me].alive > 0) {
			if (!player_piece(me, ChooseOption(Game.shapes))) {
				netint4 data[4];

				Players[me].alive = 0;
				if (lastadd == me) msg_add("\\%d" MSG_GAME_KILL_1_1,
					Players[me].team > 7 ? 7 : Players[me].team);
				else msg_add("\\%d" MSG_GAME_KILL_2_1,
					Players[lastadd].team > 7 ? 7 : Players[lastadd].team,
					Players[lastadd].name);
				if (Game.type == GT_classicTwo)
					SendPacket(me, NP_argghhh, sizeof(lastadd), &lastadd);
				window_msg_status(me);
				changed = 1;
			} //die
			else {
				status_draw(me, Players[me].score);
				if (spied) {
					SendPacket(me, NP_newPiece, sizeof(Players[me].curShape), &Players[me].curShape);
				} //send new piece
			}
		} //new piece
		while (gameStatus == 2) {
			for (i = 1; i < MAX_SCREENS; i++)
				if (Players[i].alive > 0 && window[i].shown)
					changed |= player_draw(i);
			if (changed) {
				if (!paused) status_tick();
				refresh();
				changed = 0;
			} //screen update
			{
			short playercount = 0;
			for (i = 1; i < MAX_SCREENS; i++)
				if (Players[i].alive >= 0) playercount++;
			if (playercount < 1) gameStatus = 0;
			}
			switch (WaitMyEvent(&event, EM_any)) {
			case E_alarm:
				if (!paused && Players[me].alive > 0)
					if (!player_move(me, -1, 0)) //move down
						gameStatus = 1; //new piece
					else
						if (spied) SendPacket(me, NP_down, 0, NULL);
				break;
			case E_key:
				handle_key(event.u.key);
				break;
			case E_net:
				handle_net(event.u.net);
				break;
			case E_lostConn:
				goto gameOver;
			} //handle event
			if (paused != oldPaused) {
				if (paused) {
					PauseTime();
					pauseTimeLeft = SetITimer(0, 0);
				}
				else {
					SetITimer(Game.speed, pauseTimeLeft);
					ResumeTime();
				}
				oldPaused = paused;
			} //(un)pause
		} //game loop
		Players[me].score.score++;
		game_clear(me);
	} //new piece loop
gameOver:
	SetITimer(0, 0);
}

int main(int argc, char **argv)
{
	char ch;

	Game.type = GT_onePlayer;
	port = DEFAULT_PORT;
	maxPlayer = 1;
	Game.initspeed = DEFAULT_INTERVAL;
	Game.gravity = 0;
	handle_setkeys(DEFAULT_KEYS);
	{
		int i;
		char *userName;

		for (i = 0; i < MAX_SCREENS; i++) {
			Players[i].alive = -1;
			Players[i].score.level = 1;
			Players[i].boardWidth = 10;
			Players[i].boardHeight = MAX_BOARD_HEIGHT;
			Players[i].boardVisible = 20;
			strcpy(Players[i].name, "???");
			player_empty(i);
		}
		if (!(userName = getenv("LOGNAME")) || !userName[0])
			if (!(userName = getenv("USER")) || !userName[0])
				userName = "Anonymous";
		strncpy(Players[0].name, userName, 16); //sizeof(Player.name)
		Players[0].name[16] = 0;
		Players[0].alive = 1;
	} //set defaults

//	if (getopt(argc, argv, "f:") == 'f')
//		handle_conffile(optarg);
//	else
	handle_conffile(CONFIG_FILE);
	while ((ch = getopt_long(
		argc, argv, "hHk:c:n:oSCap:i:l:t:", options, NULL
	)) != -1)
		handle_arg(ch, optarg);
	if (optind < argc) {
		Usage();
		exit(1);
	}
//	WriteConf();

	InitScreens();  //setup screen

	if (Game.type == GT_classicTwo) {
		spied = 1;
		InitiateConnection(hostStr, port);
		HandShake();
		maxPlayer = me;
		game_setpaused();
		game_loop();
	} //client
	else {
		int i;
		Game.seed = time(0);
		for (i = 0; i < sizeof(Game.shapes) / sizeof(Game.shapes[0]); i++)
			Game.shapes[i] = 1;
		Game.started = 2;
		me = 1;
		memcpy(&Players[me], &Players[0], sizeof(player_t));
		Players[me].team = 7;
		game_loop();
	} //singleplay
	return 0;
}

